import logging
import shutil
import sys
from logging import Logger
from typing import Any, List, Optional, TextIO, Type, TYPE_CHECKING

from pydantic import BaseModel, ConfigDict, Field, PrivateAttr

from wexample_prompt.common.prompt_context import PromptContext
from wexample_prompt.mixins.response.manager.base_prompt_response_manager_mixin import \
    BasePromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.interactive.choice_dict_prompt_response_manager_mixin import \
    ChoiceDictPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.interactive.choice_prompt_response_manager_mixin import \
    ChoicePromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.interactive.dir_picker_prompt_response_manager_mixin import \
    DirPickerPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.interactive.file_picker_prompt_response_manager_mixin import \
    FilePickerPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.interactive.progress_prompt_response_manager_mixin import \
    ProgressPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.messages.debug_prompt_response_manager_mixin import \
    DebugPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.messages.error_prompt_response_manager_mixin import \
    ErrorPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.messages.failure_prompt_response_manager_mixin import \
    FailurePromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.messages.info_prompt_response_manager_mixin import \
    InfoPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.messages.log_prompt_response_manager_mixin import \
    LogPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.messages.success_prompt_response_manager_mixin import \
    SuccessPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.messages.task_prompt_response_manager_mixin import \
    TaskPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.messages.warning_prompt_response_manager_mixin import \
    WarningPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.titles.subtitle_prompt_response_manager_mixin import \
    SubtitlePromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.titles.title_prompt_response_manager_mixin import \
    TitlePromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.data.list_prompt_response_manager_mixin import \
    ListPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.data.multiple_prompt_response_manager_mixin import \
    MultiplePromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.data.properties_prompt_response_manager_mixin import \
    PropertiesPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.data.suggestions_prompt_response_manager_mixin import \
    SuggestionsPromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.data.table_prompt_response_manager_mixin import \
    TablePromptResponseManagerMixin
from wexample_prompt.mixins.response.manager.data.tree_prompt_response_manager_mixin import \
    TreePromptResponseManagerMixin
from wexample_prompt.mixins.with_indent import WithIndent
from wexample_prompt.responses import BasePromptResponse
from wexample_prompt.themes.default.abstract_prompt_theme import AbstractPromptTheme
from wexample_prompt.themes.default.default_prompt_theme import DefaultPromptTheme

if TYPE_CHECKING:
    from wexample_prompt.responses.abstract_prompt_response import AbstractPromptResponse


class IoManager(
    BaseModel,
    WithIndent,
    BasePromptResponseManagerMixin,
    ChoiceDictPromptResponseManagerMixin,
    # Data
    ListPromptResponseManagerMixin,
    MultiplePromptResponseManagerMixin,
    PropertiesPromptResponseManagerMixin,
    SuggestionsPromptResponseManagerMixin,
    TablePromptResponseManagerMixin,
    TreePromptResponseManagerMixin,
    # Interactive
    ChoicePromptResponseManagerMixin,
    DirPickerPromptResponseManagerMixin,
    FilePickerPromptResponseManagerMixin,
    ProgressPromptResponseManagerMixin,
    # Messages
    DebugPromptResponseManagerMixin,
    InfoPromptResponseManagerMixin,
    LogPromptResponseManagerMixin,
    SuccessPromptResponseManagerMixin,
    TaskPromptResponseManagerMixin,
    WarningPromptResponseManagerMixin,
    FailurePromptResponseManagerMixin,
    ErrorPromptResponseManagerMixin,
    # Titles
    TitlePromptResponseManagerMixin,
    SubtitlePromptResponseManagerMixin,

):
    """Manager for handling I/O operations in the prompt system."""

    model_config = ConfigDict(arbitrary_types_allowed=True)

    theme: AbstractPromptTheme = Field(
        default_factory=DefaultPromptTheme,
        description="Theme for customizing colors and styles"
    )
    log_level: int = Field(
        default=logging.INFO,
        description="Minimum log level for file logging"
    )
    log_file: Optional[str] = Field(
        default=None,
        description="Path to log file. If None, file logging is disabled"
    )
    terminal_width: int = Field(
        default_factory=lambda: shutil.get_terminal_size().columns,
        description="Width of the terminal"
    )
    _logger: Optional[Logger] = PrivateAttr(default=None)
    _log_file_handler: Optional[TextIO] = PrivateAttr(default=None)
    _instance_count: int = PrivateAttr(default=0)
    _tty_width: int = PrivateAttr(default_factory=lambda: shutil.get_terminal_size().columns)
    _stdout: TextIO = PrivateAttr(default_factory=lambda: sys.stdout)
    _stdin: TextIO = PrivateAttr(default_factory=lambda: sys.stdin)
    _last_context: Optional[str] = PrivateAttr(default=None)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._tty_width = shutil.get_terminal_size().columns
        self._stdout = sys.stdout
        self._stdin = sys.stdin
        self._last_context = None
        self._setup_logger()

    def _setup_logger(self):
        """Set up logging configuration."""
        self._logger = logging.getLogger(__name__)
        self._logger.setLevel(self.log_level)

        # Clear any existing handlers
        self._logger.handlers.clear()

        if self.log_file:
            self._log_file_handler = logging.FileHandler(self.log_file)
            self._logger.addHandler(self._log_file_handler)

    def __del__(self):
        """Clean up resources."""
        if self._log_file_handler:
            self._log_file_handler.close()

    @classmethod
    def get_response_types(cls) -> List[Type["AbstractPromptResponse"]]:
        from wexample_prompt.responses.base_prompt_response import BasePromptResponse
        from wexample_prompt.responses.titles.title_prompt_response import TitlePromptResponse
        from wexample_prompt.responses.titles.subtitle_prompt_response import SubtitlePromptResponse
        from wexample_prompt.responses.messages.log_prompt_response import LogPromptResponse
        from wexample_prompt.responses.messages.info_prompt_response import InfoPromptResponse
        from wexample_prompt.responses.messages.debug_prompt_response import DebugPromptResponse
        from wexample_prompt.responses.messages.success_prompt_response import SuccessPromptResponse
        from wexample_prompt.responses.messages.task_prompt_response import TaskPromptResponse
        from wexample_prompt.responses.messages.warning_prompt_response import WarningPromptResponse
        from wexample_prompt.responses.messages.failure_prompt_response import FailurePromptResponse
        from wexample_prompt.responses.messages.error_prompt_response import ErrorPromptResponse
        from wexample_prompt.responses.interactive.choice_prompt_response import ChoicePromptResponse
        from wexample_prompt.responses.interactive.choice_dict_prompt_response import ChoiceDictPromptResponse
        from wexample_prompt.responses.interactive.dir_picker_prompt_response import DirPickerPromptResponse
        from wexample_prompt.responses.interactive.file_picker_prompt_response import FilePickerPromptResponse
        from wexample_prompt.responses.interactive.progress_prompt_response import ProgressPromptResponse
        from wexample_prompt.responses.data.list_prompt_response import ListPromptResponse
        from wexample_prompt.responses.data.multiple_prompt_response import MultiplePromptResponse
        from wexample_prompt.responses.data.properties_prompt_response import PropertiesPromptResponse
        from wexample_prompt.responses.data.suggestions_prompt_response import SuggestionsPromptResponse
        from wexample_prompt.responses.data.table_prompt_response import TablePromptResponse
        from wexample_prompt.responses.data.tree_prompt_response import TreePromptResponse

        return [
            BasePromptResponse,
            # Data
            ListPromptResponse,
            MultiplePromptResponse,
            PropertiesPromptResponse,
            SuggestionsPromptResponse,
            TablePromptResponse,
            TreePromptResponse,
            # Interactive
            ChoicePromptResponse,
            ChoiceDictPromptResponse,
            DirPickerPromptResponse,
            FilePickerPromptResponse,
            ProgressPromptResponse,
            # Messages
            LogPromptResponse,
            InfoPromptResponse,
            DebugPromptResponse,
            SuccessPromptResponse,
            TaskPromptResponse,
            WarningPromptResponse,
            FailurePromptResponse,
            ErrorPromptResponse,
            # Titles
            TitlePromptResponse,
            SubtitlePromptResponse,
        ]

    def create_context(self, indentation: Optional[int] = 0) -> PromptContext:
        """Create a context with current indentation and terminal width."""
        return PromptContext(
            indentation=indentation or self.log_indent,
            terminal_width=self.terminal_width,
        )

    def print_responses(self, responses: List[BasePromptResponse]) -> None:
        for response in responses:
            self.print_response(response)

    def print_response(self, response: BasePromptResponse) -> None:
        """Print a response using its own context."""
        # If response has a context but no indentation, set it
        if response.context.indentation == 0 and self.log_indent > 0:
            response.context.indentation = self.log_indent

        response.print(output=self._stdout)

    def get_input(self, prompt: str = "") -> str:
        return input(prompt)

    def print(self, message: Any, **kwargs: Any) -> None:
        # Convert message to response if it's not already one
        if isinstance(message, BasePromptResponse):
            response = message
        else:
            from wexample_prompt.responses.messages.info_prompt_response import InfoPromptResponse
            response = InfoPromptResponse.create_info(str(message))

        self.print_response(response)

    def update_terminal_width(self) -> None:
        self._tty_width = shutil.get_terminal_size().columns



"""Response for displaying and handling choice prompts."""
from typing import Any, List, Optional, Dict, Union, Type, TYPE_CHECKING

from InquirerPy import inquirer
from InquirerPy.base.control import Choice
from InquirerPy.utils import InquirerPyDefault, InquirerPySessionResult
from pydantic import Field, ConfigDict

from wexample_prompt.common.color_manager import ColorManager
from wexample_prompt.common.prompt_context import PromptContext
from wexample_prompt.common.prompt_response_line import PromptResponseLine
from wexample_prompt.common.prompt_response_segment import PromptResponseSegment
from wexample_prompt.enums.terminal_color import TerminalColor
from wexample_prompt.enums.text_style import TextStyle
from wexample_prompt.responses.interactive.abstract_interactive_prompt_response import AbstractInteractivePromptResponse

if TYPE_CHECKING:
    from wexample_prompt.example.abstract_response_example import AbstractResponseExample


class ChoicePromptResponse(AbstractInteractivePromptResponse):
    """Response for displaying a list of choices and getting user selection."""

    # Instance variables
    choices: List[Union[str, Choice]] = Field(default_factory=list)
    default: Optional["InquirerPyDefault"] = None
    inquirer_kwargs: Dict[str, Any] = Field(default_factory=dict)
    question_text: str = Field(default="")  # Stockage du texte original de la question

    # Pydantic configuration
    model_config = ConfigDict(arbitrary_types_allowed=True)

    @classmethod
    def create_choice(
        cls,
        question: str,
        choices: List[Any],
        context: Optional[PromptContext] = None,
        default: Optional["InquirerPyDefault"] = None,
        abort: Optional[str] = "> Abort",
        color: Optional[TerminalColor] = None,
        **kwargs: Any
    ) -> 'ChoicePromptResponse':
        lines = []

        # Add the question line with specified color (or blue as default) and bold style
        color = color or TerminalColor.BLUE
        lines.append(
            PromptResponseLine(segments=[
                PromptResponseSegment(
                    text=ColorManager.colorize(question, color),
                    styles=[TextStyle.BOLD]
                )
            ])
        )

        # Add each choice with a modern arrow and styling
        choices_all = choices.copy()
        if abort:
            choices_all.append(Choice(value=None, name=abort))

        for i, choice in enumerate(choices_all):
            # Handle both simple values and Choice objects
            choice_text = choice.name if isinstance(choice, Choice) else str(choice)

            lines.append(
                PromptResponseLine(segments=[
                    # Arrow indicator with cyan color
                    PromptResponseSegment(
                        text=ColorManager.colorize(f"  {i + 1}. → ", TerminalColor.CYAN)
                    ),
                    # Choice text with gray color and dim style
                    PromptResponseSegment(
                        text=ColorManager.colorize(choice_text, TerminalColor.WHITE),
                        styles=[TextStyle.DIM]
                    )
                ])
            )

        # Create default context if none provided
        if context is None:
            context = PromptContext()

        return cls(
            lines=lines,
            context=context,
            choices=choices_all,
            default=default,
            inquirer_kwargs=kwargs,
            question_text=question  # Stocker le texte original
        )

    def execute(self) -> "InquirerPySessionResult":
        """Execute the choice prompt and get user selection."""
        return inquirer.select(
            message=self.question_text,
            choices=self.choices,
            default=self.default,
            **self.inquirer_kwargs
        ).execute()

    @classmethod
    def get_example_class(cls) -> Type["AbstractResponseExample"]:
        """Get the example class for choice prompts."""
        from wexample_prompt.example.response.interactive.choice_example import ChoiceExample
        return ChoiceExample



"""Base prompt response class."""
import sys
from typing import List, Dict, Any, Optional, Type, TextIO, TYPE_CHECKING

from pydantic import Field

from wexample_prompt.responses.abstract_prompt_response import AbstractPromptResponse
from wexample_prompt.enums.message_type import MessageType
from wexample_prompt.enums.response_type import ResponseType
from wexample_prompt.enums.verbosity_level import VerbosityLevel

if TYPE_CHECKING:
    from wexample_prompt.common.prompt_context import PromptContext
    from wexample_prompt.example.abstract_response_example import AbstractResponseExample
    from wexample_prompt.common.prompt_response_line import PromptResponseLine


class BasePromptResponse(AbstractPromptResponse):
    """Base class for all prompt responses."""

    @classmethod
    def get_example_class(cls) -> Type["AbstractResponseExample"]:
        """Get the example class for this response type.

        Returns:
            Type: The example class
        """
        from wexample_prompt.example.response.base_example import BaseExample
        return BaseExample

    @classmethod
    def create_base(
        cls,
        lines: List["PromptResponseLine"],
        context: Optional["PromptContext"] = None,
        response_type: ResponseType = ResponseType.PLAIN,
        message_type: MessageType = MessageType.LOG,
        metadata: Dict[str, Any] = None,
        verbosity_level: Optional[VerbosityLevel] = None,
        **kwargs
    ) -> "BasePromptResponse":
        """Create a base prompt response.

        Args:
            lines: List of response lines
            context: Optional prompt context
            response_type: Response type
            message_type: Message type
            metadata: Optional metadata
            verbosity_level: Optional verbosity level
            **kwargs: Additional keyword arguments

        Returns:
            BasePromptResponse: A new base prompt response
        """
        return cls(
            lines=lines,
            context=context,
            response_type=response_type,
            message_type=message_type,
            metadata=metadata or {},
            verbosity_level=verbosity_level or VerbosityLevel.DEFAULT
        )

    @classmethod
    def create_from_text(
        cls,
        text: str,
        verbosity_level: VerbosityLevel,
        context: "PromptContext",
        **kwargs
    ) -> 'BasePromptResponse':
        """Create a base prompt response from text.

        Args:
            text: Text to create response from
            verbosity_level: Verbosity level for this response
            context: Prompt context
            **kwargs: Additional keyword arguments

        Returns:
            BasePromptResponse: A new base prompt response
        """
        from wexample_prompt.common.prompt_response_line import PromptResponseLine
        from wexample_prompt.common.prompt_response_segment import PromptResponseSegment

        return cls(
            lines=[
                PromptResponseLine(segments=[
                    PromptResponseSegment(text=text)
                ])
            ],
            context=context,
            verbosity_level=verbosity_level,
            **kwargs
        )

    def print(
        self,
        output: TextIO = None,
        **kwargs
    ) -> None:
        """Print the response.

        Args:
            output: Output stream to print to
            **kwargs: Additional keyword arguments
        """
        rendered = self.render()
        if rendered:
            print(rendered, file=output or sys.stdout, end="\n")

        # Exit if fatal
        if self.context and self.context.fatal:
            sys.exit(1)

    def _on_fatal(self):
        """Handle fatal errors."""
        import sys
        sys.exit(self.context.exit_code)

    def append(self, other: "BasePromptResponse") -> "BasePromptResponse":
        """Combine this response with another.

        Args:
            other: Other response to append

        Returns:
            BasePromptResponse: Combined response
        """
        return self.__class__.create_base(
            lines=self.lines + other.lines,
            context=self.context,
            response_type=self.response_type,
            message_type=self.message_type,
            metadata={**self.metadata, **other.metadata}
        )



from typing import ClassVar, Type, TYPE_CHECKING, Optional

from wexample_prompt.responses.messages.base_message_response import BaseMessageResponse

if TYPE_CHECKING:
    from wexample_prompt.enums.message_type import MessageType
    from wexample_prompt.common.prompt_context import PromptContext
    from wexample_prompt.example.abstract_response_example import AbstractResponseExample
    from wexample_prompt.enums.terminal_color import TerminalColor


class InfoPromptResponse(BaseMessageResponse):
    """Response for info messages."""
    
    SYMBOL: ClassVar[str] = "ℹ"
    
    @classmethod
    def create_info(
        cls: "InfoPromptResponse",
        message: str,
        context: "PromptContext" = None,
        color: Optional["TerminalColor"] = None,
        **kwargs
    ) -> "InfoPromptResponse":
        from wexample_prompt.enums.terminal_color import TerminalColor

        return cls._create_symbol_message(
            text=message,
            context=context,
            color=color or TerminalColor.BLUE
        )

    @classmethod
    def get_message_type(cls) -> "MessageType":
        from wexample_prompt.enums.message_type import MessageType

        return MessageType.INFO

    @classmethod
    def get_example_class(cls) -> Type["AbstractResponseExample"]:
        """Get the example class for info messages."""
        from wexample_prompt.example.response.messages.info_example import InfoExample
        return InfoExample
